import sys

import django
from unipath import Path
from django.conf import settings
from django.core.management import execute_from_command_line

BASE_DIR = Path(__file__).ancestor(2)

print(BASE_DIR)

sys.path.append(BASE_DIR.absolute())


settings.configure(
    DATABASES={
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': BASE_DIR.child('db.sqlite3')
        }
    },
    INSTALLED_APPS=[
        'django_pg_commands',
    ],
    PG_COMMAND_TOOLS=BASE_DIR.child('configuration.json')
)

django.setup()

execute_from_command_line(sys.argv)



import json

from django.test import TestCase

from .utils import read_file


class UtilsTest(TestCase):
    def setUp(self):
        self.data = json.loads(open('configuration.json').read())

    def test_read_file(self):
        self.assertDictEqual(self.data, read_file())



import psycopg2
import json

from django.conf import settings
from django.core.management import CommandError


CONFIGURATION_FILE = getattr(settings, 'PG_COMMAND_TOOLS', None)

def read_file():
    """
    this method reads the file than describe the server credencials
    """
    if CONFIGURATION_FILE is None:
        raise CommandError('You need specified PG_COMMAND_TOOLS constant in your settings file.')
    try:
        data = json.load(open(CONFIGURATION_FILE))
    except FileNotFoundError:
        raise CommandError('The file %s doesn\'t exists' % CONFIGURATION_FILE)
    except ValueError:
        raise CommandError('The file isn\'t a valid format')
    return data

def get_connection_from_dict(info):
    """
    this method make the connection
    """
    try:
        conn = psycopg2.connect(
            host=info['host'],
            user=info['user'],
            password=info['password'],
            database=info['dbname']
        )
    except psycopg2.Error:
        raise CommandError('The database credencials are not valid')
    return conn

def get_connections():
    """
    Returns the connections for both databases
    """
    data = read_file()
    to_file = data['to']
    from_file = data['from']

    to_conn = get_connection_from_dict(to_file)
    from_conn = get_connection_from_dict(from_file)
    return [from_conn, to_conn]


def execute_command(conn, command):
    """
    Execute a query and write and the database
    """
    cursor = conn.cursor()
    try:
        cursor.execute(command)
        conn.commit()
        return True
    except psycopg2.DatabaseError:
        conn.rollback()
        return False

def execute_query(conn, query):
    """
    Execute a query and return the data
    """
    cursor = conn.cursor()
    cursor.execute(query)
    columns = [column[0] for column in cursor.description]
    results = []
    for row in cursor.fetchall():
        results.append(dict(zip(columns, row)))
    return results



import os
import sys
import django_pg_commands

from setuptools import setup, find_packages


if sys.argv[-1] == 'publish':
    # if os.system("pip freeze | grep wheel"):
    #     print("wheel not installed.\nUse `pip install wheel`.\nExiting.")
    #     sys.exit()
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/*")
    print("You probably want to also tag the version now:")
    print("  git tag -a %s -m 'version %s'" % (django_pg_commands.__version__, django_pg_commands.__version__))
    print("  git push --tags")
    sys.exit()

setup(
    name='django_pg_commands',
    version=django_pg_commands.__version__,
    description="Collections of command for postgreSQL with django",
    long_description=open('readme.md').read(),
    keywords=['postgreSQL', 'django'],
    author='Franz Emil Eulate Chacolla',
    author_email='franzemil35@gmail.com',
    url='http://github.com/franzemil/django_pg_commands',
    license='MIT',
    packages=find_packages(exclude=['docs']),
    install_requires=[
        'inquirer',
        'psycopg2'
    ],
    include_package_data=True,
    zip_safe=False,
)



import sys

import django
from unipath import Path
from django.conf import settings
from django.core.management import execute_from_command_line

BASE_DIR = Path(__file__).ancestor(2)

print(BASE_DIR)

sys.path.append(BASE_DIR.absolute())


settings.configure(
    DATABASES={
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': BASE_DIR.child('db.sqlite3')
        }
    },
    INSTALLED_APPS=[
        'django_pg_commands',
    ],
    PG_COMMAND_TOOLS=BASE_DIR.child('configuration.json')
)

django.setup()


args = [sys.argv[0], 'test']
# Current module (``tests``) and its submodules.
test_cases = '.'


# # Allow accessing test options from the command line.
offset = 1
try:
    sys.argv[1]
except IndexError:
    pass
else:
    option = sys.argv[1].startswith('-')
    if not option:
        test_cases = sys.argv[1]
        offset = 2


args.append('django_pg_commands.tests')
# # ``verbosity`` can be overwritten from command line.
# args.append('--verbosity=2')
# args.extend(sys.argv[offset:])

execute_from_command_line(args)


# coding=utf-8
import difflib

import inquirer

from django.core.management import BaseCommand, CommandError
from django.db import connections
from django_pg_commands.utils import get_connections, execute_command, execute_query


class Command(BaseCommand):
    """
    Sync the functions beetween two databases
    """

    help = 'It\'s responsive to sync the function beeten to databases'
    schema = 'public'
    to_conn = None
    from_conn = None

    def add_arguments(self, parser):
        parser.add_argument(
            '--schema',
            dest='schema',
            nargs=1,
            required=True,
            help='Schema\'s name'
        )

    def handle(self, *args, **options):
        schema = options.get('schema', None)

        if schema:
            self.schema = schema[0]

        [self.from_conn, self.to_conn] = get_connections()

        self.stdout.write("connection created ...")

        [to_funcs, from_funcs] = [self.get_functions(self.to_conn), self.get_functions(self.from_conn)]
        function_to_migrate = []

        for _ in from_funcs:
            n_fun = self.compare_functions(_, to_funcs)
            if n_fun is not None:
                if n_fun['definition'] != _['definition']:
                    sm = self.inline_diff(n_fun['definition'], _['definition'])
                    _['type'] = '[MODIFICADO] ' + _['name'] + self.print_args(_['arg_names']) + ')'
                    function_to_migrate.append(_)
            else:
                _['type'] = '[NUEVO] ' + _['name'] + '(' + self.print_args(_['arg_names']) + ')'
                function_to_migrate.append(_)

        if len(function_to_migrate) == 0:
            self.stdout.write("No se presentan Cambios")
            return

        questions = [
            inquirer.Checkbox(
                'functions',
                message="Que funcion desea motificar",
                choices=[x['type'] for x in function_to_migrate],
            ),
        ]

        answers = inquirer.prompt(questions)['functions']

        functions_to_migrate = list(filter(lambda x: x['type'] in answers, function_to_migrate))

        for item in functions_to_migrate:
            self.update_function(item, self.to_conn)


    def get_functions(self, conn):
        """
        Returns all function from a connection and scheme
        """
        query = """
          SELECT n.nspname AS schema, proname AS name, proargnames AS arg_names,
          t.typname AS return_type, d.description, pg_get_functiondef(p.oid) as definition
          FROM pg_proc p
          INNER JOIN pg_type t on p.prorettype = t.oid
          LEFT JOIN pg_description d on p.oid = d.objoid
          INNER JOIN pg_namespace n on n.oid = p.pronamespace
          WHERE n.nspname = '%s'
        """ % self.schema
        return execute_query(conn, query)

    def compare_functions(self, function, list_functions):
        """
        Verify if a function was changed
        """
        args = function['arg_names'] if function['arg_names'] else []

        existe = list(filter(lambda x: function['name'] == x['name'] and len(
            x['arg_names'] if x['arg_names'] is not None else []) == len(args), list_functions))

        if len(existe) == 0:
            return None
        return existe[0]

    def print_args(self, args):
        """
        Print the function's arguments
        """
        if args is None:
            return ''
        str_args = ','.join(args)
        if len(str_args) > 60:
            return str_args[:60] + '...'
        return str_args

    def inline_diff(self, a, b):
        """
        Return de diferences between two text
        """
        matcher = difflib.SequenceMatcher(None, a, b)

        def process_tag(tag, i1, i2, j1, j2):
            if tag == 'replace':
                return '{' + matcher.a[i1:i2] + ' -> ' + matcher.b[j1:j2] + '}'
            if tag == 'delete':
                return '{- ' + matcher.a[i1:i2] + '}'
            if tag == 'equal':
                return matcher.a[i1:i2]
            if tag == 'insert':
                return '{+ ' + matcher.b[j1:j2] + '}'

        return ''.join(process_tag(*t) for t in matcher.get_opcodes())

    def update_function(self, function, conn):
        """
        Update a function 
        """
        self.stdout.write('*' * 50)
        self.stdout.write('Syncying functions: %s' % function['name'])
        if execute_command(conn, function['definition']):
            self.stdout.write('The function: %s, was synced' % function['name'])
        else:
            self.stdout.write('There\'s a problem with function: %s' % function['name'])
        self.stdout.write('*' * 50)